/*
 * DA8xx pin multiplexing configuration
 *
 * Copyright (c) 2008 MontaVista Software, Inc. <source@mvista.com>
 *
 * Based on arch/arm/mach-davinci/mux_cfg.c:
 * Copyright (c) 2007 MontaVista Software, Inc.
 *
 * This file is licensed under the terms of the GNU General Public License
 * version 2. This program is licensed "as is" without any warranty of any
 * kind, whether express or implied.
 */

#include <linux/module.h>
#include <linux/init.h>

#include <asm/hardware.h>

#include <asm/arch/mux.h>

static unsigned long pinmux_in_use[20];

const struct pin_config __initdata_or_module da8xx_pinmux[] = {
/*
 *	       description		mux	mode	mux	dbg
 *					reg	offset	mode
 */
DA8XX_MUX_CFG("GPIO7_14",		0,	0,	1,	0)
DA8XX_MUX_CFG("RTCK",			0,	0,	8,	0)
DA8XX_MUX_CFG("GPIO7_15",		0,	4,	1,	0)
DA8XX_MUX_CFG("EMU_0",			0,	4,	8,	0)
DA8XX_MUX_CFG("EMB_SDCKE",		0,	8,	1,	0)
DA8XX_MUX_CFG("EMB_CLK_GLUE",		0,	12,	1,	0)
DA8XX_MUX_CFG("EMB_CLK",		0,	12,	2,	0)
DA8XX_MUX_CFG("NEMB_CS_0",		0,	16,	1,	0)
DA8XX_MUX_CFG("NEMB_CAS",		0,	20,	1,	0)
DA8XX_MUX_CFG("NEMB_RAS",		0,	24,	1,	0)
DA8XX_MUX_CFG("NEMB_WE",		0,	28,	1,	0)
DA8XX_MUX_CFG("EMB_BA_1",		1,	0,	1,	0)
DA8XX_MUX_CFG("EMB_BA_0",		1,	4,	1,	0)
DA8XX_MUX_CFG("EMB_A_0",		1,	8,	1,	0)
DA8XX_MUX_CFG("EMB_A_1",		1,	12,	1,	0)
DA8XX_MUX_CFG("EMB_A_2",		1,	16,	1,	0)
DA8XX_MUX_CFG("EMB_A_3",		1,	20,	1,	0)
DA8XX_MUX_CFG("EMB_A_4",		1,	24,	1,	0)
DA8XX_MUX_CFG("EMB_A_5",		1,	28,	1,	0)
DA8XX_MUX_CFG("GPIO7_0",		1,	0,	8,	0)
DA8XX_MUX_CFG("GPIO7_1",		1,	4,	8,	0)
DA8XX_MUX_CFG("GPIO7_2",		1,	8,	8,	0)
DA8XX_MUX_CFG("GPIO7_3",		1,	12,	8,	0)
DA8XX_MUX_CFG("GPIO7_4",		1,	16,	8,	0)
DA8XX_MUX_CFG("GPIO7_5",		1,	20,	8,	0)
DA8XX_MUX_CFG("GPIO7_6",		1,	24,	8,	0)
DA8XX_MUX_CFG("GPIO7_7",		1,	28,	8,	0)
DA8XX_MUX_CFG("EMB_A_6",		2,	0,	1,	0)
DA8XX_MUX_CFG("EMB_A_7",		2,	4,	1,	0)
DA8XX_MUX_CFG("EMB_A_8",		2,	8,	1,	0)
DA8XX_MUX_CFG("EMB_A_9",		2,	12,	1,	0)
DA8XX_MUX_CFG("EMB_A_10",		2,	16,	1,	0)
DA8XX_MUX_CFG("EMB_A_11",		2,	20,	1,	0)
DA8XX_MUX_CFG("EMB_A_12",		2,	24,	1,	0)
DA8XX_MUX_CFG("EMB_D_31",		2,	28,	1,	0)
DA8XX_MUX_CFG("GPIO7_8",		2,	0,	8,	0)
DA8XX_MUX_CFG("GPIO7_9",		2,	4,	8,	0)
DA8XX_MUX_CFG("GPIO7_10",		2,	8,	8,	0)
DA8XX_MUX_CFG("GPIO7_11",		2,	12,	8,	0)
DA8XX_MUX_CFG("GPIO7_12",		2,	16,	8,	0)
DA8XX_MUX_CFG("GPIO7_13",		2,	20,	8,	0)
DA8XX_MUX_CFG("GPIO3_13",		2,	24,	8,	0)
DA8XX_MUX_CFG("EMB_D_30",		3,	0,	1,	0)
DA8XX_MUX_CFG("EMB_D_29",		3,	4,	1,	0)
DA8XX_MUX_CFG("EMB_D_28",		3,	8,	1,	0)
DA8XX_MUX_CFG("EMB_D_27",		3,	12,	1,	0)
DA8XX_MUX_CFG("EMB_D_26",		3,	16,	1,	0)
DA8XX_MUX_CFG("EMB_D_25",		3,	20,	1,	0)
DA8XX_MUX_CFG("EMB_D_24",		3,	24,	1,	0)
DA8XX_MUX_CFG("EMB_D_23",		3,	28,	1,	0)
DA8XX_MUX_CFG("EMB_D_22",		4,	0,	1,	0)
DA8XX_MUX_CFG("EMB_D_21",		4,	4,	1,	0)
DA8XX_MUX_CFG("EMB_D_20",		4,	8,	1,	0)
DA8XX_MUX_CFG("EMB_D_19",		4,	12,	1,	0)
DA8XX_MUX_CFG("EMB_D_18",		4,	16,	1,	0)
DA8XX_MUX_CFG("EMB_D_17",		4,	20,	1,	0)
DA8XX_MUX_CFG("EMB_D_16",		4,	24,	1,	0)
DA8XX_MUX_CFG("NEMB_WE_DQM_3",		4,	28,	1,	0)
DA8XX_MUX_CFG("NEMB_WE_DQM_2",		5,	0,	1,	0)
DA8XX_MUX_CFG("EMB_D_0",		5,	4,	1,	0)
DA8XX_MUX_CFG("EMB_D_1",		5,	8,	1,	0)
DA8XX_MUX_CFG("EMB_D_2",		5,	12,	1,	0)
DA8XX_MUX_CFG("EMB_D_3",		5,	16,	1,	0)
DA8XX_MUX_CFG("EMB_D_4",		5,	20,	1,	0)
DA8XX_MUX_CFG("EMB_D_5",		5,	24,	1,	0)
DA8XX_MUX_CFG("EMB_D_6",		5,	28,	1,	0)
DA8XX_MUX_CFG("GPIO6_0",		5,	4,	8,	0)
DA8XX_MUX_CFG("GPIO6_1",		5,	8,	8,	0)
DA8XX_MUX_CFG("GPIO6_2",		5,	12,	8,	0)
DA8XX_MUX_CFG("GPIO6_3",		5,	16,	8,	0)
DA8XX_MUX_CFG("GPIO6_4",		5,	20,	8,	0)
DA8XX_MUX_CFG("GPIO6_5",		5,	24,	8,	0)
DA8XX_MUX_CFG("GPIO6_6",		5,	28,	8,	0)
DA8XX_MUX_CFG("EMB_D_7",		6,	0,	1,	0)
DA8XX_MUX_CFG("EMB_D_8",		6,	4,	1,	0)
DA8XX_MUX_CFG("EMB_D_9",		6,	8,	1,	0)
DA8XX_MUX_CFG("EMB_D_10",		6,	12,	1,	0)
DA8XX_MUX_CFG("EMB_D_11",		6,	16,	1,	0)
DA8XX_MUX_CFG("EMB_D_12",		6,	20,	1,	0)
DA8XX_MUX_CFG("EMB_D_13",		6,	24,	1,	0)
DA8XX_MUX_CFG("EMB_D_14",		6,	28,	1,	0)
DA8XX_MUX_CFG("GPIO6_7",		6,	0,	8,	0)
DA8XX_MUX_CFG("GPIO6_8",		6,	4,	8,	0)
DA8XX_MUX_CFG("GPIO6_9",		6,	8,	8,	0)
DA8XX_MUX_CFG("GPIO6_10",		6,	12,	8,	0)
DA8XX_MUX_CFG("GPIO6_11",		6,	16,	8,	0)
DA8XX_MUX_CFG("GPIO6_12",		6,	20,	8,	0)
DA8XX_MUX_CFG("GPIO6_13",		6,	24,	8,	0)
DA8XX_MUX_CFG("GPIO6_14",		6,	28,	8,	0)
DA8XX_MUX_CFG("EMB_D_15",		7,	0,	1,	0)
DA8XX_MUX_CFG("NEMB_WE_DQM_1",		7,	4,	1,	0)
DA8XX_MUX_CFG("NEMB_WE_DQM_0",		7,	8,	1,	0)
DA8XX_MUX_CFG("SPI0_SOMI_0",		7,	12,	1,	0)
DA8XX_MUX_CFG("SPI0_SIMO_0",		7,	16,	1,	0)
DA8XX_MUX_CFG("SPI0_CLK",		7,	20,	1,	0)
DA8XX_MUX_CFG("NSPI0_ENA",		7,	24,	1,	0)
DA8XX_MUX_CFG("NSPI0_SCS_0",		7,	28,	1,	0)
DA8XX_MUX_CFG("EQEP0I",			7,	12,	2,	0)
DA8XX_MUX_CFG("EQEP0S", 		7,	16,	2,	0)
DA8XX_MUX_CFG("EQEP1I", 		7,	20,	2,	0)
DA8XX_MUX_CFG("NUART0_CTS",		7,	24,	2,	0)
DA8XX_MUX_CFG("NUART0_RTS",		7,	28,	2,	0)
DA8XX_MUX_CFG("EQEP0A", 		7,	24,	4,	0)
DA8XX_MUX_CFG("EQEP0B", 		7,	28,	4,	0)
DA8XX_MUX_CFG("GPIO6_15",		7,	0,	8,	0)
DA8XX_MUX_CFG("GPIO5_14",		7,	4,	8,	0)
DA8XX_MUX_CFG("GPIO5_15",		7,	8,	8,	0)
DA8XX_MUX_CFG("GPIO5_0",		7,	12,	8,	0)
DA8XX_MUX_CFG("GPIO5_1",		7,	16,	8,	0)
DA8XX_MUX_CFG("GPIO5_2",		7,	20,	8,	0)
DA8XX_MUX_CFG("GPIO5_3",		7,	24,	8,	0)
DA8XX_MUX_CFG("GPIO5_4",		7,	28,	8,	0)
DA8XX_MUX_CFG("SPI1_SOMI_0",		8,	0,	1,	0)
DA8XX_MUX_CFG("SPI1_SIMO_0",		8,	4,	1,	0)
DA8XX_MUX_CFG("SPI1_CLK",		8,	8,	1,	0)
DA8XX_MUX_CFG("UART0_RXD",		8,	12,	1,	0)
DA8XX_MUX_CFG("UART0_TXD",		8,	16,	1,	0)
DA8XX_MUX_CFG("AXR1_10",		8,	20,	1,	0)
DA8XX_MUX_CFG("AXR1_11",		8,	24,	1,	0)
DA8XX_MUX_CFG("NSPI1_ENA",		8,	28,	1,	0)
DA8XX_MUX_CFG("I2C1_SCL",		8,	0,	2,	0)
DA8XX_MUX_CFG("I2C1_SDA",		8,	4,	2,	0)
DA8XX_MUX_CFG("EQEP1S", 		8,	8,	2,	0)
DA8XX_MUX_CFG("I2C0_SDA",		8,	12,	2,	0)
DA8XX_MUX_CFG("I2C0_SCL",		8,	16,	2,	0)
DA8XX_MUX_CFG("UART2_RXD",		8,	28,	2,	0)
DA8XX_MUX_CFG("TM64P0_IN12",		8,	12,	4,	0)
DA8XX_MUX_CFG("TM64P0_OUT12",		8,	16,	4,	0)
DA8XX_MUX_CFG("GPIO5_5",		8,	0,	8,	0)
DA8XX_MUX_CFG("GPIO5_6",		8,	4,	8,	0)
DA8XX_MUX_CFG("GPIO5_7",		8,	8,	8,	0)
DA8XX_MUX_CFG("GPIO5_8",		8,	12,	8,	0)
DA8XX_MUX_CFG("GPIO5_9",		8,	16,	8,	0)
DA8XX_MUX_CFG("GPIO5_10",		8,	20,	8,	0)
DA8XX_MUX_CFG("GPIO5_11",		8,	24,	8,	0)
DA8XX_MUX_CFG("GPIO5_12",		8,	28,	8,	0)
DA8XX_MUX_CFG("NSPI1_SCS_0",		9,	0,	1,	0)
DA8XX_MUX_CFG("USB0_DRVVBUS",		9,	4,	1,	0)
DA8XX_MUX_CFG("AHCLKX0",		9,	8,	1,	0)
DA8XX_MUX_CFG("ACLKX0", 		9,	12,	1,	0)
DA8XX_MUX_CFG("AFSX0",			9,	16,	1,	0)
DA8XX_MUX_CFG("AHCLKR0",		9,	20,	1,	0)
DA8XX_MUX_CFG("ACLKR0", 		9,	24,	1,	0)
DA8XX_MUX_CFG("AFSR0",			9,	28,	1,	0)
DA8XX_MUX_CFG("UART2_TXD",		9,	0,	2,	0)
DA8XX_MUX_CFG("AHCLKX2",		9,	8,	2,	0)
DA8XX_MUX_CFG("ECAP0_APWM0",		9,	12,	2,	0)
DA8XX_MUX_CFG("RMII_MHZ_50_CLK",	9,	20,	2,	0)
DA8XX_MUX_CFG("ECAP1_APWM1",		9,	24,	2,	0)
DA8XX_MUX_CFG("USB_REFCLKIN",		9,	8,	4,	0)
DA8XX_MUX_CFG("GPIO5_13",		9,	0,	8,	0)
DA8XX_MUX_CFG("GPIO4_15",		9,	4,	8,	0)
DA8XX_MUX_CFG("GPIO2_11",		9,	8,	8,	0)
DA8XX_MUX_CFG("GPIO2_12",		9,	12,	8,	0)
DA8XX_MUX_CFG("GPIO2_13",		9,	16,	8,	0)
DA8XX_MUX_CFG("GPIO2_14",		9,	20,	8,	0)
DA8XX_MUX_CFG("GPIO2_15",		9,	24,	8,	0)
DA8XX_MUX_CFG("GPIO3_12",		9,	28,	8,	0)
DA8XX_MUX_CFG("AMUTE0", 		10,	0,	1,	0)
DA8XX_MUX_CFG("AXR0_0", 		10,	4,	1,	0)
DA8XX_MUX_CFG("AXR0_1", 		10,	8,	1,	0)
DA8XX_MUX_CFG("AXR0_2", 		10,	12,	1,	0)
DA8XX_MUX_CFG("AXR0_3", 		10,	16,	1,	0)
DA8XX_MUX_CFG("AXR0_4", 		10,	20,	1,	0)
DA8XX_MUX_CFG("AXR0_5", 		10,	24,	1,	0)
DA8XX_MUX_CFG("AXR0_6", 		10,	28,	1,	0)
DA8XX_MUX_CFG("RMII_TXD_0",		10,	4,	2,	0)
DA8XX_MUX_CFG("RMII_TXD_1",		10,	8,	2,	0)
DA8XX_MUX_CFG("RMII_TXEN",		10,	12,	2,	0)
DA8XX_MUX_CFG("RMII_CRS_DV",		10,	16,	2,	0)
DA8XX_MUX_CFG("RMII_RXD_0",		10,	20,	2,	0)
DA8XX_MUX_CFG("RMII_RXD_1",		10,	24,	2,	0)
DA8XX_MUX_CFG("RMII_RXER",		10,	28,	2,	0)
DA8XX_MUX_CFG("AFSR2",			10,	4,	4,	0)
DA8XX_MUX_CFG("ACLKX2", 		10,	8,	4,	0)
DA8XX_MUX_CFG("AXR2_3", 		10,	12,	4,	0)
DA8XX_MUX_CFG("AXR2_2", 		10,	16,	4,	0)
DA8XX_MUX_CFG("AXR2_1", 		10,	20,	4,	0)
DA8XX_MUX_CFG("AFSX2",			10,	24,	4,	0)
DA8XX_MUX_CFG("ACLKR2", 		10,	28,	4,	0)
DA8XX_MUX_CFG("NRESETOUT",		10,	0,	8,	0)
DA8XX_MUX_CFG("GPIO3_0",		10,	4,	8,	0)
DA8XX_MUX_CFG("GPIO3_1",		10,	8,	8,	0)
DA8XX_MUX_CFG("GPIO3_2",		10,	12,	8,	0)
DA8XX_MUX_CFG("GPIO3_3",		10,	16,	8,	0)
DA8XX_MUX_CFG("GPIO3_4",		10,	20,	8,	0)
DA8XX_MUX_CFG("GPIO3_5",		10,	24,	8,	0)
DA8XX_MUX_CFG("GPIO3_6",		10,	28,	8,	0)
DA8XX_MUX_CFG("AXR0_7", 		11,	0,	1,	0)
DA8XX_MUX_CFG("AXR0_8", 		11,	4,	1,	0)
DA8XX_MUX_CFG("UART1_RXD",		11,	8,	1,	0)
DA8XX_MUX_CFG("UART1_TXD",		11,	12,	1,	0)
DA8XX_MUX_CFG("AXR0_11",		11,	16,	1,	0)
DA8XX_MUX_CFG("AHCLKX1",		11,	20,	1,	0)
DA8XX_MUX_CFG("ACLKX1", 		11,	24,	1,	0)
DA8XX_MUX_CFG("AFSX1",			11,	28,	1,	0)
DA8XX_MUX_CFG("MDIO_CLK",		11,	0,	2,	0)
DA8XX_MUX_CFG("MDIO_D", 		11,	4,	2,	0)
DA8XX_MUX_CFG("AXR0_9", 		11,	8,	2,	0)
DA8XX_MUX_CFG("AXR0_10",		11,	12,	2,	0)
DA8XX_MUX_CFG("EPWM0B", 		11,	20,	2,	0)
DA8XX_MUX_CFG("EPWM0A", 		11,	24,	2,	0)
DA8XX_MUX_CFG("EPWMSYNCI",		11,	28,	2,	0)
DA8XX_MUX_CFG("AXR2_0", 		11,	16,	4,	0)
DA8XX_MUX_CFG("EPWMSYNC0",		11,	28,	4,	0)
DA8XX_MUX_CFG("GPIO3_7",		11,	0,	8,	0)
DA8XX_MUX_CFG("GPIO3_8",		11,	4,	8,	0)
DA8XX_MUX_CFG("GPIO3_9",		11,	8,	8,	0)
DA8XX_MUX_CFG("GPIO3_10",		11,	12,	8,	0)
DA8XX_MUX_CFG("GPIO3_11",		11,	16,	8,	0)
DA8XX_MUX_CFG("GPIO3_14",		11,	20,	8,	0)
DA8XX_MUX_CFG("GPIO3_15",		11,	24,	8,	0)
DA8XX_MUX_CFG("GPIO4_10",		11,	28,	8,	0)
DA8XX_MUX_CFG("AHCLKR1",		12,	0,	1,	0)
DA8XX_MUX_CFG("ACLKR1", 		12,	4,	1,	0)
DA8XX_MUX_CFG("AFSR1",			12,	8,	1,	0)
DA8XX_MUX_CFG("AMUTE1", 		12,	12,	1,	0)
DA8XX_MUX_CFG("AXR1_0", 		12,	16,	1,	0)
DA8XX_MUX_CFG("AXR1_1", 		12,	20,	1,	0)
DA8XX_MUX_CFG("AXR1_2", 		12,	24,	1,	0)
DA8XX_MUX_CFG("AXR1_3", 		12,	28,	1,	0)
DA8XX_MUX_CFG("ECAP2_APWM2",		12,	4,	2,	0)
DA8XX_MUX_CFG("EHRPWMGLUETZ",		12,	12,	2,	0)
DA8XX_MUX_CFG("EQEP1A", 		12,	28,	2,	0)
DA8XX_MUX_CFG("GPIO4_11",		12,	0,	8,	0)
DA8XX_MUX_CFG("GPIO4_12",		12,	4,	8,	0)
DA8XX_MUX_CFG("GPIO4_13",		12,	8,	8,	0)
DA8XX_MUX_CFG("GPIO4_14",		12,	12,	8,	0)
DA8XX_MUX_CFG("GPIO4_0",		12,	16,	8,	0)
DA8XX_MUX_CFG("GPIO4_1",		12,	20,	8,	0)
DA8XX_MUX_CFG("GPIO4_2",		12,	24,	8,	0)
DA8XX_MUX_CFG("GPIO4_3",		12,	28,	8,	0)
DA8XX_MUX_CFG("AXR1_4", 		13,	0,	1,	0)
DA8XX_MUX_CFG("AXR1_5", 		13,	4,	1,	0)
DA8XX_MUX_CFG("AXR1_6", 		13,	8,	1,	0)
DA8XX_MUX_CFG("AXR1_7", 		13,	12,	1,	0)
DA8XX_MUX_CFG("AXR1_8", 		13,	16,	1,	0)
DA8XX_MUX_CFG("AXR1_9", 		13,	20,	1,	0)
DA8XX_MUX_CFG("EMA_D_0",		13,	24,	1,	0)
DA8XX_MUX_CFG("EMA_D_1",		13,	28,	1,	0)
DA8XX_MUX_CFG("EQEP1B", 		13,	0,	2,	0)
DA8XX_MUX_CFG("EPWM2B", 		13,	4,	2,	0)
DA8XX_MUX_CFG("EPWM2A", 		13,	8,	2,	0)
DA8XX_MUX_CFG("EPWM1B", 		13,	12,	2,	0)
DA8XX_MUX_CFG("EPWM1A", 		13,	16,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_0",		13,	24,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_1",		13,	28,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_0",		13,	24,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_1",		13,	28,	4,	0)
DA8XX_MUX_CFG("GPIO4_4",		13,	0,	8,	0)
DA8XX_MUX_CFG("GPIO4_5",		13,	4,	8,	0)
DA8XX_MUX_CFG("GPIO4_6",		13,	8,	8,	0)
DA8XX_MUX_CFG("GPIO4_7",		13,	12,	8,	0)
DA8XX_MUX_CFG("GPIO4_8",		13,	16,	8,	0)
DA8XX_MUX_CFG("GPIO4_9",		13,	20,	8,	0)
DA8XX_MUX_CFG("GPIO0_0",		13,	24,	8,	0)
DA8XX_MUX_CFG("GPIO0_1",		13,	28,	8,	0)
DA8XX_MUX_CFG("EMA_D_2",		14,	0,	1,	0)
DA8XX_MUX_CFG("EMA_D_3",		14,	4,	1,	0)
DA8XX_MUX_CFG("EMA_D_4",		14,	8,	1,	0)
DA8XX_MUX_CFG("EMA_D_5",		14,	12,	1,	0)
DA8XX_MUX_CFG("EMA_D_6",		14,	16,	1,	0)
DA8XX_MUX_CFG("EMA_D_7",		14,	20,	1,	0)
DA8XX_MUX_CFG("EMA_D_8",		14,	24,	1,	0)
DA8XX_MUX_CFG("EMA_D_9",		14,	28,	1,	0)
DA8XX_MUX_CFG("MMCSD_DAT_2",		14,	0,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_3",		14,	4,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_4",		14,	8,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_5",		14,	12,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_6",		14,	16,	2,	0)
DA8XX_MUX_CFG("MMCSD_DAT_7",		14,	20,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_8",		14,	24,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_9",		14,	28,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_2",		14,	0,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_3",		14,	4,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_4",		14,	8,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_5",		14,	12,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_6",		14,	16,	4,	0)
DA8XX_MUX_CFG("UHPI_HD_7",		14,	20,	4,	0)
DA8XX_MUX_CFG("LCD_D_8",		14,	24,	4,	0)
DA8XX_MUX_CFG("LCD_D_9",		14,	28,	4,	0)
DA8XX_MUX_CFG("GPIO0_2",		14,	0,	8,	0)
DA8XX_MUX_CFG("GPIO0_3",		14,	4,	8,	0)
DA8XX_MUX_CFG("GPIO0_4",		14,	8,	8,	0)
DA8XX_MUX_CFG("GPIO0_5",		14,	12,	8,	0)
DA8XX_MUX_CFG("GPIO0_6",		14,	16,	8,	0)
DA8XX_MUX_CFG("GPIO0_7",		14,	20,	8,	0)
DA8XX_MUX_CFG("GPIO0_8",		14,	24,	8,	0)
DA8XX_MUX_CFG("GPIO0_9",		14,	28,	8,	0)
DA8XX_MUX_CFG("EMA_D_10",		15,	0,	1,	0)
DA8XX_MUX_CFG("EMA_D_11",		15,	4,	1,	0)
DA8XX_MUX_CFG("EMA_D_12",		15,	8,	1,	0)
DA8XX_MUX_CFG("EMA_D_13",		15,	12,	1,	0)
DA8XX_MUX_CFG("EMA_D_14",		15,	16,	1,	0)
DA8XX_MUX_CFG("EMA_D_15",		15,	20,	1,	0)
DA8XX_MUX_CFG("EMA_A_0",		15,	24,	1,	0)
DA8XX_MUX_CFG("EMA_A_1",		15,	28,	1,	0)
DA8XX_MUX_CFG("UHPI_HD_10",		15,	0,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_11",		15,	4,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_12",		15,	8,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_13",		15,	12,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_14",		15,	16,	2,	0)
DA8XX_MUX_CFG("UHPI_HD_15",		15,	20,	2,	0)
DA8XX_MUX_CFG("LCD_D_7",		15,	24,	2,	0)
DA8XX_MUX_CFG("MMCSD_CLK",		15,	28,	2,	0)
DA8XX_MUX_CFG("LCD_D_10",		15,	0,	4,	0)
DA8XX_MUX_CFG("LCD_D_11",		15,	4,	4,	0)
DA8XX_MUX_CFG("LCD_D_12",		15,	8,	4,	0)
DA8XX_MUX_CFG("LCD_D_13",		15,	12,	4,	0)
DA8XX_MUX_CFG("LCD_D_14",		15,	16,	4,	0)
DA8XX_MUX_CFG("LCD_D_15",		15,	20,	4,	0)
DA8XX_MUX_CFG("UHPI_HCNTL0",		15,	28,	4,	0)
DA8XX_MUX_CFG("GPIO0_10",		15,	0,	8,	0)
DA8XX_MUX_CFG("GPIO0_11",		15,	4,	8,	0)
DA8XX_MUX_CFG("GPIO0_12",		15,	8,	8,	0)
DA8XX_MUX_CFG("GPIO0_13",		15,	12,	8,	0)
DA8XX_MUX_CFG("GPIO0_14",		15,	16,	8,	0)
DA8XX_MUX_CFG("GPIO0_15",		15,	20,	8,	0)
DA8XX_MUX_CFG("GPIO1_0",		15,	24,	8,	0)
DA8XX_MUX_CFG("GPIO1_1",		15,	28,	8,	0)
DA8XX_MUX_CFG("EMA_A_2",		16,	0,	1,	0)
DA8XX_MUX_CFG("EMA_A_3",		16,	4,	1,	0)
DA8XX_MUX_CFG("EMA_A_4",		16,	8,	1,	0)
DA8XX_MUX_CFG("EMA_A_5",		16,	12,	1,	0)
DA8XX_MUX_CFG("EMA_A_6",		16,	16,	1,	0)
DA8XX_MUX_CFG("EMA_A_7",		16,	20,	1,	0)
DA8XX_MUX_CFG("EMA_A_8",		16,	24,	1,	0)
DA8XX_MUX_CFG("EMA_A_9",		16,	28,	1,	0)
DA8XX_MUX_CFG("MMCSD_CMD",		16,	0,	2,	0)
DA8XX_MUX_CFG("LCD_D_6",		16,	4,	2,	0)
DA8XX_MUX_CFG("LCD_D_3",		16,	8,	2,	0)
DA8XX_MUX_CFG("LCD_D_2",		16,	12,	2,	0)
DA8XX_MUX_CFG("LCD_D_1",		16,	16,	2,	0)
DA8XX_MUX_CFG("LCD_D_0",		16,	20,	2,	0)
DA8XX_MUX_CFG("LCD_PCLK",		16,	24,	2,	0)
DA8XX_MUX_CFG("LCD_HSYNC",		16,	28,	2,	0)
DA8XX_MUX_CFG("UHPI_HCNTL1",		16,	0,	4,	0)
DA8XX_MUX_CFG("GPIO1_2",		16,	0,	8,	0)
DA8XX_MUX_CFG("GPIO1_3",		16,	4,	8,	0)
DA8XX_MUX_CFG("GPIO1_4",		16,	8,	8,	0)
DA8XX_MUX_CFG("GPIO1_5",		16,	12,	8,	0)
DA8XX_MUX_CFG("GPIO1_6",		16,	16,	8,	0)
DA8XX_MUX_CFG("GPIO1_7",		16,	20,	8,	0)
DA8XX_MUX_CFG("GPIO1_8",		16,	24,	8,	0)
DA8XX_MUX_CFG("GPIO1_9",		16,	28,	8,	0)
DA8XX_MUX_CFG("EMA_A_10",		17,	0,	1,	0)
DA8XX_MUX_CFG("EMA_A_11",		17,	4,	1,	0)
DA8XX_MUX_CFG("EMA_A_12",		17,	8,	1,	0)
DA8XX_MUX_CFG("EMA_BA_1",		17,	12,	1,	0)
DA8XX_MUX_CFG("EMA_BA_0",		17,	16,	1,	0)
DA8XX_MUX_CFG("EMA_CLK",		17,	20,	1,	0)
DA8XX_MUX_CFG("EMA_SDCKE",		17,	24,	1,	0)
DA8XX_MUX_CFG("NEMA_CAS",		17,	28,	1,	0)
DA8XX_MUX_CFG("LCD_VSYNC",		17,	0,	2,	0)
DA8XX_MUX_CFG("NLCD_AC_ENB_CS", 	17,	4,	2,	0)
DA8XX_MUX_CFG("LCD_MCLK",		17,	8,	2,	0)
DA8XX_MUX_CFG("LCD_D_5",		17,	12,	2,	0)
DA8XX_MUX_CFG("LCD_D_4",		17,	16,	2,	0)
DA8XX_MUX_CFG("OBSCLK", 		17,	20,	2,	0)
DA8XX_MUX_CFG("NEMA_CS_4",		17,	28,	2,	0)
DA8XX_MUX_CFG("UHPI_HHWIL",		17,	12,	4,	0)
DA8XX_MUX_CFG("AHCLKR2",		17,	20,	4,	0)
DA8XX_MUX_CFG("GPIO1_10",		17,	0,	8,	0)
DA8XX_MUX_CFG("GPIO1_11",		17,	4,	8,	0)
DA8XX_MUX_CFG("GPIO1_12",		17,	8,	8,	0)
DA8XX_MUX_CFG("GPIO1_13",		17,	12,	8,	0)
DA8XX_MUX_CFG("GPIO1_14",		17,	16,	8,	0)
DA8XX_MUX_CFG("GPIO1_15",		17,	20,	8,	0)
DA8XX_MUX_CFG("GPIO2_0",		17,	24,	8,	0)
DA8XX_MUX_CFG("GPIO2_1",		17,	28,	8,	0)
DA8XX_MUX_CFG("NEMA_RAS",		18,	0,	1,	0)
DA8XX_MUX_CFG("NEMA_WE",		18,	4,	1,	0)
DA8XX_MUX_CFG("NEMA_CS_0",		18,	8,	1,	0)
DA8XX_MUX_CFG("NEMA_CS_2",		18,	12,	1,	0)
DA8XX_MUX_CFG("NEMA_CS_3",		18,	16,	1,	0)
DA8XX_MUX_CFG("NEMA_OE",		18,	20,	1,	0)
DA8XX_MUX_CFG("NEMA_WE_DQM_1",		18,	24,	1,	0)
DA8XX_MUX_CFG("NEMA_WE_DQM_0",		18,	28,	1,	0)
DA8XX_MUX_CFG("NEMA_CS_5",		18,	0,	2,	0)
DA8XX_MUX_CFG("UHPI_HRNW",		18,	4,	2,	0)
DA8XX_MUX_CFG("NUHPI_HAS",		18,	8,	2,	0)
DA8XX_MUX_CFG("NUHPI_HCS",		18,	12,	2,	0)
DA8XX_MUX_CFG("NUHPI_HDS1",		18,	20,	2,	0)
DA8XX_MUX_CFG("NUHPI_HDS2",		18,	24,	2,	0)
DA8XX_MUX_CFG("NUHPI_HINT",		18,	28,	2,	0)
DA8XX_MUX_CFG("AXR0_12",		18,	4,	4,	0)
DA8XX_MUX_CFG("AMUTE2", 		18,	16,	4,	0)
DA8XX_MUX_CFG("AXR0_13",		18,	20,	4,	0)
DA8XX_MUX_CFG("AXR0_14",		18,	24,	4,	0)
DA8XX_MUX_CFG("AXR0_15",		18,	28,	4,	0)
DA8XX_MUX_CFG("GPIO2_2",		18,	0,	8,	0)
DA8XX_MUX_CFG("GPIO2_3",		18,	4,	8,	0)
DA8XX_MUX_CFG("GPIO2_4",		18,	8,	8,	0)
DA8XX_MUX_CFG("GPIO2_5",		18,	12,	8,	0)
DA8XX_MUX_CFG("GPIO2_6",		18,	16,	8,	0)
DA8XX_MUX_CFG("GPIO2_7",		18,	20,	8,	0)
DA8XX_MUX_CFG("GPIO2_8",		18,	24,	8,	0)
DA8XX_MUX_CFG("GPIO2_9",		18,	28,	8,	0)
DA8XX_MUX_CFG("EMA_WAIT_0",		19,	0,	1,	0)
DA8XX_MUX_CFG("NUHPI_HRDY",		19,	0,	2,	0)
DA8XX_MUX_CFG("GPIO2_10",		19,	0,	8,	0)
};

static const short da8xx_emif25_pins[] = {
	DA8XX_EMA_D_0, DA8XX_EMA_D_1, DA8XX_EMA_D_2, DA8XX_EMA_D_3,
	DA8XX_EMA_D_4, DA8XX_EMA_D_5, DA8XX_EMA_D_6, DA8XX_EMA_D_7,
	DA8XX_EMA_D_8, DA8XX_EMA_D_9, DA8XX_EMA_D_10, DA8XX_EMA_D_11,
	DA8XX_EMA_D_12, DA8XX_EMA_D_13, DA8XX_EMA_D_14, DA8XX_EMA_D_15,
	DA8XX_EMA_A_0, DA8XX_EMA_A_1, DA8XX_EMA_A_2, DA8XX_EMA_A_3,
	DA8XX_EMA_A_4, DA8XX_EMA_A_5, DA8XX_EMA_A_6, DA8XX_EMA_A_7,
	DA8XX_EMA_A_8, DA8XX_EMA_A_9, DA8XX_EMA_A_10, DA8XX_EMA_A_11,
	DA8XX_EMA_A_12, DA8XX_EMA_BA_0, DA8XX_EMA_BA_1, DA8XX_EMA_CLK,
	DA8XX_EMA_SDCKE, DA8XX_NEMA_CS_4, DA8XX_NEMA_CS_5, DA8XX_NEMA_WE,
	DA8XX_NEMA_CS_0, DA8XX_NEMA_CS_2, DA8XX_NEMA_CS_3, DA8XX_NEMA_OE,
	DA8XX_NEMA_WE_DQM_1, DA8XX_NEMA_WE_DQM_0, DA8XX_EMA_WAIT_0,
	-1
};

static const short da8xx_spi0_pins[] = {
	DA8XX_SPI0_SOMI_0, DA8XX_SPI0_SIMO_0, DA8XX_SPI0_CLK, DA8XX_NSPI0_ENA,
	DA8XX_NSPI0_SCS_0,
	-1
};

static const short da8xx_mmc_sd_pins[] = {
	DA8XX_MMCSD_DAT_0, DA8XX_MMCSD_DAT_1, DA8XX_MMCSD_DAT_2,
	DA8XX_MMCSD_DAT_3, DA8XX_MMCSD_DAT_4, DA8XX_MMCSD_DAT_5,
	DA8XX_MMCSD_DAT_6, DA8XX_MMCSD_DAT_7, DA8XX_MMCSD_CLK,
	DA8XX_MMCSD_CMD,
	-1
};

static const short da8xx_uart0_pins[] = {
	DA8XX_NUART0_CTS, DA8XX_NUART0_RTS, DA8XX_UART0_RXD, DA8XX_UART0_TXD,
	-1
};

static const short da8xx_usb20_pins[] = {
	DA8XX_USB0_DRVVBUS, DA8XX_USB_REFCLKIN,
	-1
};

static const short da8xx_usb11_pins[] = {
	DA8XX_USB_REFCLKIN,
	-1
};

static const short da8xx_uhpi_pins[] = {
	DA8XX_UHPI_HD_0, DA8XX_UHPI_HD_1, DA8XX_UHPI_HD_2, DA8XX_UHPI_HD_3,
	DA8XX_UHPI_HD_4, DA8XX_UHPI_HD_5, DA8XX_UHPI_HD_6, DA8XX_UHPI_HD_7,
	DA8XX_UHPI_HD_8, DA8XX_UHPI_HD_9, DA8XX_UHPI_HD_10, DA8XX_UHPI_HD_11,
	DA8XX_UHPI_HD_12, DA8XX_UHPI_HD_13, DA8XX_UHPI_HD_14, DA8XX_UHPI_HD_15,
	DA8XX_UHPI_HCNTL0, DA8XX_UHPI_HCNTL1, DA8XX_UHPI_HHWIL, DA8XX_UHPI_HRNW,
	DA8XX_NUHPI_HAS, DA8XX_NUHPI_HCS, DA8XX_NUHPI_HDS1, DA8XX_NUHPI_HDS2,
	DA8XX_NUHPI_HINT, DA8XX_NUHPI_HRDY,
	-1
};

static const short da8xx_cpgmac_pins[] = {
	DA8XX_RMII_TXD_0, DA8XX_RMII_TXD_1, DA8XX_RMII_TXEN, DA8XX_RMII_CRS_DV,
	DA8XX_RMII_RXD_0, DA8XX_RMII_RXD_1, DA8XX_RMII_RXER, DA8XX_MDIO_CLK,
	DA8XX_MDIO_D,
	-1
};

static const short da8xx_emif3c_pins[] = {
	DA8XX_EMB_SDCKE, DA8XX_EMB_CLK_GLUE, DA8XX_EMB_CLK, DA8XX_NEMB_CS_0,
	DA8XX_NEMB_CAS, DA8XX_NEMB_RAS, DA8XX_NEMB_WE, DA8XX_EMB_BA_1,
	DA8XX_EMB_BA_0, DA8XX_EMB_A_0, DA8XX_EMB_A_1, DA8XX_EMB_A_2,
	DA8XX_EMB_A_3, DA8XX_EMB_A_4, DA8XX_EMB_A_5, DA8XX_EMB_A_6,
	DA8XX_EMB_A_7, DA8XX_EMB_A_8, DA8XX_EMB_A_9, DA8XX_EMB_A_10,
	DA8XX_EMB_A_11, DA8XX_EMB_A_12, DA8XX_NEMB_WE_DQM_3,
	DA8XX_NEMB_WE_DQM_2, DA8XX_EMB_D_0, DA8XX_EMB_D_1, DA8XX_EMB_D_2,
	DA8XX_EMB_D_3, DA8XX_EMB_D_4, DA8XX_EMB_D_5, DA8XX_EMB_D_6,
	DA8XX_EMB_D_7, DA8XX_EMB_D_8, DA8XX_EMB_D_9, DA8XX_EMB_D_10,
	DA8XX_EMB_D_11, DA8XX_EMB_D_12, DA8XX_EMB_D_13, DA8XX_EMB_D_14,
	DA8XX_EMB_D_15, DA8XX_EMB_D_16, DA8XX_EMB_D_17, DA8XX_EMB_D_18,
	DA8XX_EMB_D_19, DA8XX_EMB_D_20, DA8XX_EMB_D_21, DA8XX_EMB_D_22,
	DA8XX_EMB_D_23, DA8XX_EMB_D_24, DA8XX_EMB_D_25, DA8XX_EMB_D_26,
	DA8XX_EMB_D_27, DA8XX_EMB_D_28, DA8XX_EMB_D_29, DA8XX_EMB_D_30,
	DA8XX_EMB_D_31, DA8XX_NEMB_WE_DQM_1, DA8XX_NEMB_WE_DQM_0,
	-1
};

static const short da8xx_mcasp0_pins[] = {
	DA8XX_AHCLKX0, DA8XX_ACLKX0, DA8XX_AFSX0,
	DA8XX_AHCLKR0, DA8XX_ACLKR0, DA8XX_AFSR0, DA8XX_AMUTE0,
	DA8XX_AXR0_0, DA8XX_AXR0_1, DA8XX_AXR0_2, DA8XX_AXR0_3,
	DA8XX_AXR0_4, DA8XX_AXR0_5, DA8XX_AXR0_6, DA8XX_AXR0_7,
	DA8XX_AXR0_8, DA8XX_AXR0_9, DA8XX_AXR0_10, DA8XX_AXR0_11,
	DA8XX_AXR0_12, DA8XX_AXR0_13, DA8XX_AXR0_14, DA8XX_AXR0_15,
	-1
};

static const short da8xx_mcasp1_pins[] = {
	DA8XX_AHCLKX1, DA8XX_ACLKX1, DA8XX_AFSX1,
	DA8XX_AHCLKR1, DA8XX_ACLKR1, DA8XX_AFSR1, DA8XX_AMUTE1,
	DA8XX_AXR1_0, DA8XX_AXR1_1, DA8XX_AXR1_2, DA8XX_AXR1_3,
	DA8XX_AXR1_4, DA8XX_AXR1_5, DA8XX_AXR1_6, DA8XX_AXR1_7,
	DA8XX_AXR1_8, DA8XX_AXR1_9, DA8XX_AXR1_10, DA8XX_AXR1_11,
	-1
};

static const short da8xx_mcasp2_pins[] = {
	DA8XX_AHCLKX2, DA8XX_ACLKX2, DA8XX_AFSX2,
	DA8XX_AHCLKR2, DA8XX_ACLKR2, DA8XX_AFSR2, DA8XX_AMUTE2,
	DA8XX_AXR2_0, DA8XX_AXR2_1, DA8XX_AXR2_2, DA8XX_AXR2_3,
	-1
};

static const short da8xx_spi1_pins[] = {
	DA8XX_SPI1_SOMI_0, DA8XX_SPI1_SIMO_0, DA8XX_SPI1_CLK, DA8XX_NSPI1_ENA,
	DA8XX_NSPI1_SCS_0,
	-1
};

static const short da8xx_i2c_pins[] = {
	DA8XX_I2C0_SDA, DA8XX_I2C0_SCL, DA8XX_I2C1_SCL, DA8XX_I2C1_SDA,
	-1
};

static const short da8xx_uart1_pins[] = {
	DA8XX_UART1_RXD, DA8XX_UART1_TXD,
	-1
};

static const short da8xx_uart2_pins[] = {
	DA8XX_UART2_RXD, DA8XX_UART2_TXD,
	-1
};

static const short da8xx_lcdcntl_pins[] = {
	DA8XX_LCD_D_0, DA8XX_LCD_D_1, DA8XX_LCD_D_2, DA8XX_LCD_D_3,
	DA8XX_LCD_D_4, DA8XX_LCD_D_5, DA8XX_LCD_D_6, DA8XX_LCD_D_7,
	DA8XX_LCD_D_8, DA8XX_LCD_D_9, DA8XX_LCD_D_10, DA8XX_LCD_D_11,
	DA8XX_LCD_D_12, DA8XX_LCD_D_13, DA8XX_LCD_D_14, DA8XX_LCD_D_15,
	DA8XX_LCD_PCLK, DA8XX_LCD_HSYNC, DA8XX_LCD_VSYNC, DA8XX_NLCD_AC_ENB_CS,
	DA8XX_LCD_MCLK,
	-1
};

static const short da8xx_pwm_pins[] = {
	DA8XX_ECAP0_APWM0, DA8XX_ECAP1_APWM1, DA8XX_EPWM0B, DA8XX_EPWM0A,
	DA8XX_EPWMSYNCI, DA8XX_EPWMSYNC0, DA8XX_ECAP2_APWM2, DA8XX_EHRPWMGLUETZ,
	DA8XX_EPWM2B, DA8XX_EPWM2A, DA8XX_EPWM1B, DA8XX_EPWM1A,
	-1
};

static const short da8xx_ecap_pins[] = {
	DA8XX_ECAP0_APWM0, DA8XX_ECAP1_APWM1, DA8XX_ECAP2_APWM2,
	-1
};

static const short da8xx_eqep_pins[] = {
	DA8XX_EQEP0I, DA8XX_EQEP0S, DA8XX_EQEP0A, DA8XX_EQEP0B, DA8XX_EQEP1I,
	DA8XX_EQEP1S, DA8XX_EQEP1A, DA8XX_EQEP1B,
	-1
};

const short *da8xx_psc0_pins[DA8XX_LPSC0_GEM + 1] = {
	[DA8XX_LPSC0_EMIF25]	= da8xx_emif25_pins,
	[DA8XX_LPSC0_SPI0]	= da8xx_spi0_pins,
	[DA8XX_LPSC0_MMC_SD]	= da8xx_mmc_sd_pins,
	[DA8XX_LPSC0_UART0]	= da8xx_uart0_pins,
};

const short *da8xx_psc1_pins[DA8XX_LPSC1_L3_CBA_RAM + 1] = {
	[DA8XX_LPSC1_USB20]	= da8xx_usb20_pins,
	[DA8XX_LPSC1_USB11]	= da8xx_usb11_pins,
	[DA8XX_LPSC1_UHPI]	= da8xx_uhpi_pins,
	[DA8XX_LPSC1_CPGMAC]	= da8xx_cpgmac_pins,
	[DA8XX_LPSC1_EMIF3C]	= da8xx_emif3c_pins,
	[DA8XX_LPSC1_McASP0]	= da8xx_mcasp0_pins,
	[DA8XX_LPSC1_McASP1]	= da8xx_mcasp1_pins,
	[DA8XX_LPSC1_McASP2]	= da8xx_mcasp2_pins,
	[DA8XX_LPSC1_SPI1]	= da8xx_spi1_pins,
	[DA8XX_LPSC1_I2C]	= da8xx_i2c_pins,
	[DA8XX_LPSC1_UART1]	= da8xx_uart1_pins,
	[DA8XX_LPSC1_UART2]	= da8xx_uart2_pins,
	[DA8XX_LPSC1_LCDC]	= da8xx_lcdcntl_pins,
	[DA8XX_LPSC1_PWM]	= da8xx_pwm_pins,
	[DA8XX_LPSC1_ECAP]	= da8xx_ecap_pins,
	[DA8XX_LPSC1_EQEP]	= da8xx_eqep_pins,
};

static const short *da8xx_get_pins(unsigned ctlr, unsigned id)
{
	switch (ctlr) {
	case 0:
		return id > DA8XX_LPSC0_GEM ? NULL : da8xx_psc0_pins[id];
	case 1:
		return id > DA8XX_LPSC1_L3_CBA_RAM ? NULL : da8xx_psc1_pins[id];
	}
	return NULL;
}

void __init da8xx_mux_init(void)
{
	davinci_mux_register(da8xx_pinmux, ARRAY_SIZE(da8xx_pinmux),
			     da8xx_get_pins, pinmux_in_use);
}
